import { ValidatorFn, AsyncValidatorFn, ValidationErrors } from "./directives/validators";
import { Observable } from "rxjs";
export declare type FormHooks = "change" | "blur" | "submit";
export declare const VALID = "VALID";
export declare const INVALID = "INVALID";
export declare const PENDING = "PENDING";
export declare const DISABLED = "DISABLED";
export interface AbstractControlOptions {
    validators?: ValidatorFn | ValidatorFn[] | null;
    asyncValidators?: AsyncValidatorFn | AsyncValidatorFn[] | null;
    updateOn?: "change" | "blur" | "submit";
}
export declare abstract class AbstractControl {
    validator: ValidatorFn | null;
    asyncValidator: AsyncValidatorFn | null;
    readonly value: any;
    readonly errors: ValidationErrors | null;
    readonly valueChanges: Observable<any>;
    readonly statusChanges: Observable<any>;
    readonly status: string;
    _updateOn: FormHooks;
    _asyncValidationSubscription: any;
    _parent: FormGroup;
    _onCollectionChange: () => void;
    constructor(validator: ValidatorFn | null, asyncValidator: AsyncValidatorFn | null);
    readonly parent: FormGroup;
    readonly valid: boolean;
    readonly invalid: boolean;
    readonly pending: boolean;
    readonly disabled: boolean;
    readonly enabled: boolean;
    /** @internal */
    _initObservables(): void;
    setValidators(newValidator: ValidatorFn | ValidatorFn[] | null): void;
    setAsyncValidators(newValidator: AsyncValidatorFn | AsyncValidatorFn[] | null): void;
    clearValidators(): void;
    disable(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    enable(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    setParent(parent: FormGroup): void;
    abstract _updateValue(): void;
    abstract setValue(value: any, options?: Object): void;
    abstract patchValue(value: any, options?: Object): void;
    abstract reset(value?: any, options?: Object): void;
    get(path: Array<string | number> | string): AbstractControl | null;
    private _updateAncestors;
    private _runAsyncValidator;
    private _cancelExistingSubscription;
    private _setInitialStatus;
    updateValueAndValidity(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    private _runValidator;
    private _calculateStatus;
    _anyControlsHaveStatus(status: string): boolean;
    abstract _anyControls(condition: Function): boolean;
    /** @internal */
    abstract _allControlsDisabled(): boolean;
    setErrors(errors: ValidationErrors | null, opts?: {
        emitEvent?: boolean;
    }): void;
    getError(errorCode: string, path?: Array<string | number> | string): any;
    hasError(errorCode: string, path?: Array<string | number> | string): boolean;
    readonly root: AbstractControl;
    _updateControlsErrors(emitEvent: boolean): void;
    _registerOnCollectionChange(fn: () => void): void;
    _isBoxedValue(formState: any): boolean;
}
export declare class FormControl extends AbstractControl {
    /** @internal */
    _onChange: Function[];
    /** @internal */
    _pendingValue: any;
    /** @internal */
    _pendingChange: any;
    constructor(formState?: any, validatorOrOpts?: ValidatorFn | ValidatorFn[] | AbstractControlOptions | null, asyncValidator?: AsyncValidatorFn | AsyncValidatorFn[] | null);
    setValue(value: any, options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
        emitModelToViewChange?: boolean;
        emitViewToModelChange?: boolean;
    }): void;
    patchValue(value: any, options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
        emitModelToViewChange?: boolean;
        emitViewToModelChange?: boolean;
    }): void;
    reset(formState?: any, options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    /**
     * @internal
     */
    _updateValue(): void;
    /**
     * @internal
     */
    _anyControls(condition: Function): boolean;
    /**
     * @internal
     */
    _allControlsDisabled(): boolean;
    /**
     * Register a listener for change events.
     *
     * @param fn The method that is called when the value changes
     */
    registerOnChange(fn: Function): void;
    /**
     * @internal
     */
    _clearChangeFns(): void;
    /**
     * Register a listener for disabled events.
     *
     * @param fn The method that is called when the disabled status changes.
     */
    registerOnDisabledChange(fn: (isDisabled: boolean) => void): void;
    /**
     * @internal
     */
    _forEachChild(cb: Function): void;
    private _applyFormState;
}
export declare class FormGroup extends AbstractControl {
    controls: {
        [key: string]: AbstractControl;
    };
    bind?: any;
    constructor(controls: {
        [key: string]: AbstractControl;
    }, validatorOrOpts?: ValidatorFn | ValidatorFn[] | AbstractControlOptions | null, asyncValidator?: AsyncValidatorFn | AsyncValidatorFn[] | null, bind?: any);
    registerControl(name: string, control: AbstractControl): AbstractControl;
    addControl(name: string, control: AbstractControl): void;
    removeControl(name: string): void;
    setControl(name: string, control: AbstractControl): void;
    contains(controlName: string): boolean;
    setValue(value: {
        [key: string]: any;
    }, options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    patchValue(value: {
        [key: string]: any;
    }, options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    reset(value?: any, options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    getRawValue(): any;
    /** @internal */
    _throwIfControlMissing(name: string): void;
    /** @internal */
    _forEachChild(cb: (v: any, k: string) => void): void;
    /** @internal */
    _setUpControls(): void;
    /** @internal */
    _updateValue(): void;
    /** @internal */
    _anyControls(condition: Function): boolean;
    /** @internal */
    _reduceValue(): any;
    /** @internal */
    _reduceChildren(initValue: any, fn: Function): any;
    /** @internal */
    _allControlsDisabled(): boolean;
    /** @internal */
    _checkAllValuesPresent(value: any): void;
}
